# Copyright (C) 2003, 2004
# Emmanuel Saracco <esaracco@noos.fr>
# Easter-eggs <http://www.easter-eggs.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330,
# Boston, MA 02111-1307, USA.

do '../web-lib.pl';
&init_config();

use POSIX;
use File::Basename;

# freshclam configuration
my %freshclam_config = ();

# clamav configuration
my %clamav_config = ();

# clamav version this module support
my $supported_versions = '0.67';

# week days
my @days = (
    $text{'SUNDAY'},
    $text{'MONDAY'},
    $text{'TUESDAY'},
    $text{'WEDNESDAY'},
    $text{'THURSDAY'},
    $text{'FRIDAY'},
    $text{'SATURDAY'}
  );

# clamav_check_config ()
# IN: -
# OUT: -
#
# check if the module config is ok. if not, display a warning
# and exit
# 
sub clamav_main_check_config
{
  my $ok = 0;
  my $msg = '';

  &clamav_trim_config ();
  
  if (not &clamav_check_version ())
  {
     $msg = $text{'MSG_BAD_CLAMAV_VERSION'};
  }
  elsif ($config{'clamav_init_script'} eq '' or
    ! -x $config{'clamav_init_script'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_DAEMON'};
  }
  elsif ($config{'clamav_daemon_name'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_DAEMON_NAME'};
  }
  elsif (not &clamav_bin_exist ('clamscan'))
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMSCAN'};
  }
  elsif ($config{'clamav_clamav_log'} eq '' or
   ! -f $config{'clamav_clamav_log'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_LOG'};
  }
  elsif ($config{'clamav_clamav_conf'} eq '' or
    ! -f $config{'clamav_clamav_conf'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_CLAMAV_CONF'};
  }
  elsif ($config{'clamav_db1'} eq '' or
    ! -f $config{'clamav_db1'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_DB1'};
  }
  elsif ($config{'clamav_db2'} eq '' or
    ! -f $config{'clamav_db2'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_DB2'};
  }
  elsif (not &clamav_bin_exist ('freshclam'))
  {
    $msg = $text{'MSG_CONFIG_ALERT_REFRESH'};
  }
  elsif ($config{'clamav_freshclam_log'} eq '' or
    ! -f $config{'clamav_freshclam_log'})
  {
    $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_LOG'};
  }
  elsif ($config{'clamav_refresh_use_cron'} == 0)
  {
    if ($config{'clamav_freshclam_conf'} eq '' or
      ! -f $config{'clamav_freshclam_conf'})
    {
      $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_CONF'};
    }
    elsif ($config{'clamav_freshclam_init_script'} eq '' or
      ! -x $config{'clamav_freshclam_init_script'})
    {
      $msg = $text{'MSG_CONFIG_ALERT_FRESHCLAM_INIT'};
    }
    else
    {
      $ok = 1;
    }
  }
  elsif ($config{'clamav_refresh_use_cron'} eq '')
  {
    $msg = $text{'MSG_CONFIG_ALERT_USE_CRON'};
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (not $ok);
}

# clamav_bin_exist ( $ )
# IN: the binary to test
# OUT: -
#
# Test the existence and the usability (x) of a given binary
# on the server
# 
sub clamav_bin_exist ( $ )
{
  my $bin = shift;
  
  return (
    -x "/sbin/$bin" or
    -x "/usr/sbin/$bin" or
    -x "/bin/$bin" or
    -x "/usr/bin/$bin" or
    -x "/usr/local/sbin/$bin" or
    -x "/usr/local/bin/$bin" or
    -x "/usr/lib/$bin"
  );
}

# clamav_quarantine_resend_check_config ()
# IN: -
# OUT: -
#
# check if the config is ok for resending quarantined emails. 
# if not, display a warning and exit
# 
sub clamav_quarantine_resend_check_config
{
  my $ok = 0;
  my $msg = '';

  &clamav_trim_config ();
  
  if (not &clamav_bin_exist ('rsmtp') and not &clamav_bin_exist ('sendmail'))
  {
    $msg = $text{'MSG_CONFIG_ALERT_MTA'};
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (not $ok);
}

# clamav_quarantine_main_check_config ()
# IN: -
# OUT: -
#
# check if the config is ok for managing quarantined emails. 
# if not, display a warning and exit
#
sub clamav_quarantine_main_check_config
{
  my $ok = 0;

  &clamav_trim_config ();
  
  if ($config{'clamav_quarantine'} eq '' or
    ! -d $config{'clamav_quarantine'})
  {
    print qq(<p>$text{'MSG_CONFIG_ALERT_QUARANTINE'}</p>);
  }
  else
  {
    $ok = 1;
  }

  &clamav_check_config_exit ($msg) if (not $ok);
}

# clamav_signatures_check_config ()
# IN: -
# OUT: -
#
# check if the server config is ok for the signatures creation section. 
# if not, display a warning and exit
#
sub clamav_signatures_check_config
{
  &clamav_trim_config ();

  &clamav_check_config_exit ($text{'MSG_ERROR_SIGNATURES_CONFIG'})
    if (
      not &clamav_bin_exist ('strings') or 
      not &clamav_bin_exist ('hexdump')
    );
}

# clamav_check_config_exit ( $ )
# IN: Message to display
# OUT: -
#
# print a message and exit
# 
sub clamav_check_config_exit ( $ )
{
  my $msg = shift;

  print qq(<p>$msg</p>) if ($msg);
  print qq(<hr>);
  print "<a href=\"/config.cgi?$module_name\">",
    $text{'header_config'},"</a>\n";

  exit (1);
}

# clamav_trim_config ()
# IN: -
# OUT: -
#
# remove trailing and pending spaces from every module's
# config variable
# 
sub clamav_trim_config
{
  foreach $key (keys %config)
  {
    $config{$key} =~ s/^ +//g;
    $config{$key} =~ s/ +$//g;
  }
}

# clamav_html_cut ( $ $ )
# IN: the string to cut, the max length before cutting
# OUT: the cutted string with HTML <br> tags
#
# cut a string with HTML <br> tags
# 
sub clamav_html_cut ( $ $ )
{
  my ($string, $len) = @_;
  my $i = 0;
  my $new = '';

  for ($i = 0; $i < length ($string); $i++)
  {
    $new .= '<br>' if ($i > 0 and not ($i % $len));
    $new .= substr ($string, $i, 1);
  }

  return $new;
}

# clamav_check_signature ( $ )
# IN: new virus signature
# OUT: 1 => the signature begun with 00
#      2 => the signature is not between 40-200 characters
#      3 => signature is ok
#
# check the ClamAV compatibility for a given virus signature
#      
sub clamav_check_signature ( $ )
{
  my $signature = shift;
  my $ret = 3;
  

  if ($signature =~ /^00/)
  {
    $ret = 1;
  }
  elsif (length ($signature) < 40 or length ($signature) > 200)
  {
    $ret = 2;
  }

  return $ret;
}

# clamav_check_version ()
# IN: -
# OUT: boolean value
#
# return true if the installed version of clamav
# is supported by this module
# 
sub clamav_check_version
{
  (split (' ', `clamscan --version 2>&1`))[4] =~ 
    /^(\d+\.\d+).*$/;

  return $1 >= $supported_versions;
}

# clamav_get_runlevel ()
# IN: -
# OUT: the current runlevel
#
# return the current runlevel of the server system
# 
sub clamav_get_runlevel
{
  my $runlevel = '';

  $runlevel = `/sbin/runlevel`;
  $runlevel =~ s/N //;
  chop ($runlevel);

  return $runlevel;
}

# clamav_get_freshclam_daemon_settings ()
# IN: -
# OUT: check period
#
# Return the check period specified in the configuration file of
# freshclam daemon
sub clamav_get_freshclam_daemon_settings
{
  my $conf = '';
  my $ret = '';
  
  &clamav_load_freshclam_config ();
  
  $ret = $freshclam_config{'Checks'};
  $ret = 1 if $ret eq '';

  return $ret;
}

# clamav_vdb_search ( $ $ )
# IN: virus name, TRUE if we must do a strict search
# OUT: -
#
# Do a search in ClamAV database and display the result
# 
sub clamav_vdb_search ( $ $ )
{
  my ($virus, $strict, $case, $sortr) = @_;
  my $string = '';
  my $escaped = '';
  my $first = 1;

  exit (1) if ($virus =~ /;/);

  # case sensitive search?
  $case = ($case) ? ' ' : ' -i ';

  # sort results
  $sortr = ($sortr) ? ' sort |' : '';
  
  # strict check (exact match)?
  $string = ($virus ne '') ? 
    (($strict) ? 
      "sigtool --list-sigs | grep $case \"^$virus\$\" | $sortr" :
      "sigtool --list-sigs | grep $case $virus | $sortr") :
    "sigtool --list-sigs | $sortr";

  # display results
  open (H, $string);
  while (<H>)
  {
    chomp ();
    $escaped = &urlize ($_);

    if ($first)
    {
      $first = 0;
      
      print qq(<table border=1>);
      print qq(<tr $tb><th>$text{'NAME'}</th>);
      print qq(<th>Symantec</th>);
      print qq(<th>McAfee</th>);
      print qq(<th>Trend Micro</th>);
      print qq(</tr>);
    }

    $symantec_url = 
      "http://search.symantec.com/custom/us/query.html?" .
      "lk=1&rf=0&qt=$escaped";
    $mcafee_url = 
      "http://us.mcafee.com/virusInfo/default.asp?" .
      "id=alphar&SearchType=2&searchString=$escaped";
    $trend_url = 
      "http://www.trendmicro.com/vinfo/virusencyclo/default2.asp?" .
      "m=q&virus=$escaped&alt=$escaped";
      
    print qq(<tr><td $cb>$_</td>);
    print qq(<td $cb><a href="$symantec_url">$text{'MORE'}</a></td>);
    print qq(<td $cb><a href="$mcafee_url">$text{'MORE'}</a></td>);
    print qq(<td $cb><a href="$trend_url">$text{'MORE'}</a></td>);
    print qq(</tr>);
  }
  close (H);
  
  if (not $first)
  {
    print qq(</table>);
  }
  else
  {
    printf "<p>$text{'NO_RESULT'}</p>", $virus;
  }
}

# clamav_get_db_viruses_count ()
# IN: -
# OUT: Number of viruses in ClamAV database
#
# Count the number of viruses in ClamAV databases and return
# it
# 
sub clamav_get_db_viruses_count
{
  my $count = `sigtool --list-sigs | wc -l`;
  
  $count =~ s/ //g;
  
  return $count;
}

# clamav_get_freshclam_daemon_state ()
# IN: -
# OUT: true if the freshclam daemon is deactivated
#
# Return the actual state of the freshclam daemon service
# 
sub clamav_get_freshclam_daemon_state
{
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';

  $runlevel = &clamav_get_runlevel ();
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);

  # FIXME
  if ($gconfig{'os_type'} =~ /debian/)
  {
    return (! -e "/etc/network/if-up.d/$freshclambase"); 
  }
  else
  {
    return (< /etc/rc$runlevel.d/S*$freshclambase > eq '');
  }
}

# clamav_verif_refresh_method_ok ()
# IN: -
# OUT: 1 => config say to use a daemon but no daemon exist on the system
#      2 => config say to use a daemon, but a cron exist on the system
#      3 => config say tu use cron, but a daemon exist on the system
#      4 => config and system are ok
#
# Check if the refresh method choose by user in module config is
# ok according to the system
# 
sub clamav_verif_refresh_method_ok
{
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $use_cron = 0;
  
  $runlevel = &clamav_get_runlevel ();
  $use_cron = $config{'clamav_refresh_use_cron'};
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);

  # FIXME
  if ($gconfig{'os_type'} =~ /debian/)
  {
    return 1
      if (($use_cron == 0) and 
        (! -e "$freshclam"));
  }
  else
  {
   return 1
    if (($use_cron == 0) and 
      (not < /etc/rc$runlevel.d/{S,K}*$freshclambase >));
  }

  return 2
    if (($use_cron == 0) and (&clamav_get_cron_settings ('update')));

  # FIXME
  if ($gconfig{'os_type'} =~ /debian/)
  {
    return 3
      if (($use_cron == 1) and (-e < /etc/network/if-up.d/$freshclambase >));
  }
  else
  {
   return 3
    if (($use_cron == 1) and (< /etc/rc$runlevel.d/S*$freshclambase >));
  }

  return 4;
}

# clamav_scandir ( $ $ )
# IN: directory to scan, flag to indicate to use or not recursivity
# OUT: -
#
# Check a given directory and display freshclam output on the current web
# page
# 
sub clamav_scandir ( $ $ )
{
  my ($dir, $recursive) = @_;
 
  $recursive = ($recursive) ?
    ' -r ' : '';
      
  print qq(<div><pre style="background: #cccccc">);
  open (H, "clamscan $recursive $dir 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    print;
  }
  close (H);
  print qq(</pre>);
}

# clamav_get_last_db_update ()
# IN: -
# OUT: a array with main and daily db update date
#
# Check the signatures databases dates and return them in a
# array
# 
sub clamav_get_last_db_update
{
  my $main = '';
  my $daily = '';
         
  $main = strftime('%Y-%m-%d %H:%M', 
    localtime((stat($config{'clamav_db1'}))[9]))
      if (-e $config{'clamav_db1'});
  $daily = strftime('%Y-%m-%d %H:%M', 
    localtime((stat($config{'clamav_db2'}))[9]))
      if (-e $config{'clamav_db2'});
 
  return ($main, $daily);
}

# clamav_set_db_no_autoupdate ()
# IN: -
# OUT: -
#
# if user choose to not automatically update db, we stop clamd and
# deactivate cron and services
# 
sub clamav_set_db_no_autoupdate
{
  my $runlevel = '';
  my $filename = '';
  my $freshclam = '';
  my $freshclambase = '';
  my $order = 0;

  $runlevel = &clamav_get_runlevel ();
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
  system ("$freshclam stop");

  # FIXME
  if ($gconfig{'os_type'} =~ /debian/)
  {
    system ("/bin/rm -f /etc/network/if-up.d/$freshclambase");
    system ("/bin/rm -f /etc/network/if-down.d/$freshclambase");
  }
  else
  {
    $filename = < /etc/rc$runlevel.d/{K,S}*$freshclambase >;
    if (length ($filename) > 0)
    {
      $base = basename ($filename);
      $base =~ /^.([0-9]+).*$/;
      $order = $1;

      system (
        "/bin/mv -f /etc/rc$runlevel.d/S$order$freshclambase " .
        "/etc/rc$runlevel.d/K$order$freshclambase");
    }
  }
  
  &clamav_delete_cron ('update');
}

# clamav_delete_cron ( $ )
# IN: type of the cron (purge|update)
# OUT: -
#
# Delete the line associated with the given type in
# /etc/cron.d/webmin_clamav
# 
sub clamav_delete_cron ( $ )
{
  my $type = shift;
  my @lines = ();

  open (H, "/etc/cron.d/webmin_clamav");
  @lines = <H>;
  close (H);

  for (my $i = 0; $i < $#lines + 1; $i++)
  {
    splice (@lines, $i, 1) 
      if ($lines[$i] =~ /^.*#.*$type/);
  }

  open (H, ">/etc/cron.d/webmin_clamav");
  print H @lines;
  close (H);
}

# clamav_set_freshclam_daemon_settings ( $ $ )
# IN: old frequency, new frequency
# OUT: -
#
# Update the check interval frequency of the refresh for the
# freshclam daemon and restat it
# 
sub clamav_set_freshclam_daemon_settings ( $ $ )
{
  my ($oldfreq, $freq) = @_;
  my $runlevel = '';
  my $freshclam = '';
  my $freshclambase = '';

  $runlevel = &clamav_get_runlevel ();
  $freshclam = $config{'clamav_freshclam_init_script'};
  $freshclambase = basename ($freshclam);
 
 # FIXME
 if ($gconfig{'os_type'} =~ /debian/)
  {
    system ("/bin/ln -s $freshclam /etc/network/if-up.d/");
    system ("/bin/ln -s $freshclam /etc/network/if-down.d/");
  }
  else
  {
    $filename = < /etc/rc$runlevel.d/{K,S}*$freshclambase >;
    $base = basename ($filename);
    $base =~ /^.([0-9]+).*$/;
    $order = $1;

    system (
      "/bin/mv -f /etc/rc$runlevel.d/K$order$freshclambase " .
      "/etc/rc$runlevel.d/S$order$freshclambase");
  }
  
  &clamav_load_freshclam_config ();
  $freshclam_config{'Checks'} = $freq;
  &clamav_save_freshclam_config ();

  system ("$freshclam restart");
}

# clamav_save_freshclam_config ()
# IN: -
# OUT: -
#
# Save the freshclam configuration in freshclam config file
# 
sub clamav_save_freshclam_config
{
  open (H, ">$config{'clamav_freshclam_conf'}");
  foreach my $key (keys %freshclam_config)
  {
    print H "$key $freshclam_config{$key}\n";
  }
  close (H);
}

# clamav_save_global_settings ()
# IN: -
# OUT: -
#
# Save ClamAV/Freshclam settings in config files
# 
sub clamav_save_global_settings
{
  open (CLAMAV, ">$config{'clamav_clamav_conf'}");
  open (FRESHCLAM, ">$config{'clamav_freshclam_conf'}");
  foreach $key (keys %in)
  {
    $value = $key;
    if ($key =~ /^clamav.*$/)
    {
      $value =~ s/^clamav_//;
      print CLAMAV "$value $in{$key}\n";
    }
    elsif ($key =~ /^freshclam.*$/)
    {
      $value =~ s/^freshclam_//;
      print FRESHCLAM "$value $in{$key}\n";
    }
  }
  close (CLAMAV);
  close (FRESHCLAM);

  &clamav_activate_clamd ();
  if ($config{'clamav_refresh_use_cron'} eq '0')
  {
    system ("$config{'clamav_freshclam_init_script'} restart");
  }
}

# clamav_display_clamav_settings ()
# IN: -
# OUT: -
#
# Load clamav configuration file and display it
# 
sub clamav_display_clamav_settings
{
  &clamav_load_clamav_config ();

  print qq(<table border=1>);
  foreach $key (keys %clamav_config)
  {
    next if ($key eq '');
    
    if ($clamav_config{$key} eq '')
    {
      print qq(<input type="hidden" name="clamav_$key" value="">);
    }
    else
    {
      print qq(<tr>);
      print qq(<td $cb>$key</td>);
      print qq(<td><input type="text" name="clamav_$key" size=40 );
      print qq(value="$clamav_config{$key}"></td>);
      print qq(</tr>);
    }
  }
  print qq(</table>);
}

# clamav_display_freshclam_settings ()
# IN: -
# OUT: -
#
# Load freshclam configuration file and display it
# 
sub clamav_display_freshclam_settings
{
  &clamav_load_freshclam_config ();

  print qq(<table border=1>);
  foreach $key (keys %freshclam_config)
  {
    next if ($key eq '');

    if ($freshclam_config{$key} eq '')
    {
      print qq(<input type="hidden" name="freshclam_$key" value="">);
    }
    else
    {
      print qq(<tr>);
      print qq(<td $cb>$key</td>);
      print qq(<td><input type="text" name="freshclam_$key" size=40 );
      print qq(value="$freshclam_config{$key}"></td>);
      print qq(</tr>);
    }
  }
  print qq(</table>);
}

# clamav_load_clamav_config ()
# IN: -
# OUT: -
#
# Load the clamav config file in memory
# 
sub clamav_load_clamav_config
{
  %clamav_config = undef;
  
  open (H, "<$config{'clamav_clamav_conf'}");
  while (<H>)
  {
    next if /#/;

    (my $key, my $value) = split (' ');
    $clamav_config{$key} = $value;
  }
  close (H);
}

# clamav_load_freshclam_config ()
# IN: -
# OUT: -
#
# Load the freshclam config file in memory
# 
sub clamav_load_freshclam_config
{
  %freshclam_config = undef;
  
  open (H, "<$config{'clamav_freshclam_conf'}");
  while (<H>)
  {
    next if /#/;

    (my $key, my $value) = split (' ');
    $freshclam_config{$key} = $value;
  }
  close (H);
}

# clamav_set_cron_update ( $ $ )
# IN: cron hour, cron day
# OUT: -
#
# Set the cron line for the db refresh in /etc/cron.d/webmin_clamav
# 
sub clamav_set_cron_update ( $ $ )
{
  my ($hour, $day) = @_;
  my $new_line = '';
  my $datadir = '';

  $datadir = dirname ($config{'clamav_db1'});

  $day = '*' if ($day == 7);
  &clamav_delete_cron ('update');
  $new_line = 
    "$hour * * * $day root \$(freshclam --quiet " .
    "--log $config{'clamav_freshclam_log'} --datadir $datadir) #update";

  open (H, ">>/etc/cron.d/webmin_clamav");
  print H "$new_line\n";
  close (H);
}

# clamav_get_logfiles ()
# IN: -
# OUT: A array with logfiles
#
# Identify ClamAV log files and fill a array with them
#
sub clamav_get_logfiles
{
  @ret = ();

  foreach $key (%config)
  {
    push @ret, $key if ($key =~ /^.*\.log$/);
  }

  return @ret;
}

# clamav_set_cron_purge ( $ $ )
# IN: cron hour, cron day
# OUT: -
#
# Set cron line for the quarantine repository purge
# 
sub clamav_set_cron_purge ( $ $ )
{
  my ($hour, $day) = @_;
  my $new_line = '';

  $day = '*' if ($day == 7);
  &clamav_delete_cron ('purge');
  $new_line = 
    "$hour * * * $day root \$(/bin/rm -f " .
    "$config{'clamav_quarantine'}/*.{msg,log}) #purge";

  open (H, ">>/etc/cron.d/webmin_clamav");
  print H "$new_line\n";
  close (H);
}

# clamav_update_db ()
# IN: -
# OUT: -
#
# Update virus signatures databases displaying output on the current web
# page
# 
sub clamav_update_db
{
  my $proxy = '';
  my $host = $config{'clamav_proxy_host'};
  my $port = $config{'clamav_proxy_port'};
  
  $proxy = "--http-proxy=$host" . (($port) ? ":$port" : '')
    if ($host);

  print qq(<div><pre style="background: #cccccc">);;
  open (H, "freshclam $proxy 2>&1 |");
  while (<H>)
  {
    s/^*\n$/<br>/g;
    if (/load/)
    {
      print '.';
    }
    else
    {
      print;
    };
  }
  close (H);
  print qq(</pre>);
}

# clamav_get_cron_settings ( $ )
# IN: type of the cron (purge|update)
# OUT: a array representing a cron line
#
# Return a array of a cron line corresponding to the given type. if line
# is not founded, return undef
# 
sub clamav_get_cron_settings ( $ )
{
  my $type = shift;
  my @lines = ();
  my @cron_line = ();

  return undef 
    if (! -e "/etc/cron.d/webmin_clamav");

  open (H, "/etc/cron.d/webmin_clamav");
  @lines = <H>;
  close (H);
                                                                                
  foreach (@lines)
  {
    @cron_line = 
      (/^(\w+|\*)\s+(\w+|\*)\s+(\w+|\*)\s+(\w+|\*)\s+(\w+|\*)\s+(\w+)\s+(.*)\s+(#\w+)$/);
                                                                                
    return @cron_line
      if ($8 =~ /$type/);
  }
                                                                                
  return undef;
}

# clamav_freshclam_daemon_settings_table ( $ )
# IN: frequency
# OUT: a buffer to display
#
# Build a HTML table with all possible frequencies for freshclam
# daemon
# 
sub clamav_freshclam_daemon_settings_table ( $ )
{
  my $freq = shift;

  $buffer .= <<EOF;
<table border="1">
<tr $tb><td><b>$text{'FREQUENCY'}</b></td></tr>
<tr $cb>
<td><select name="freq">
EOF

  foreach (1..50)
  {
    $default = ($_ == $freq) ? ' SELECTED' : '';
    $buffer .= qq(<option value="$_"$default>$_</option>\n);
  }

$buffer .= <<EOF;
</select></td></tr>
</table>
EOF

  return $buffer;
}

# clamav_cron_settings_table ( $ $ )
# IN: default cron hour, default cron day
# OUT: a buffer to display
#
# Build a HTML table for choose hour a day for execution of refresh
# db cron
#
sub clamav_cron_settings_table
{
  my ($hour, $day) = @_;
  my $buffer = '';

$buffer .= <<EOF;
<table border="1">
<tr $tb><td><b>$text{'HOUR'}</b></td><td><b>$text{'DAY'}</b></td></tr>
<tr $cb>
<td><select name="hour">
EOF

  foreach (0..23)
  {
    $default = ($_ == $hour) ? ' SELECTED' : '';
    $buffer .= qq(<option value="$_"$default>$_</option>\n);
  }

$buffer .= <<EOF;
</select></td>
<td><select name="day">
EOF

  $default = ($day == 7) ? ' SELECTED' : '';
  $buffer .= qq(<option value="7"$default>$text{'EVERYDAY'}</option>\n);
  for (my $i = 0; $i < 7; $i++)
  {
    my $value = $days[$i];
    my $default = ($i == $day) ? ' SELECTED' : '';
                                                                                
    $buffer .= qq(<option value="$i"$default>$value</option>\n);
  }

$buffer .= <<EOF;
</select></td></tr>
</table>
EOF

  return $buffer;
}

# clamav_is_quarantine_repository_empty ()
# IN: -
# OUT: true if empty
#
# Check wether the quarantine repository is empty or not.
# 
sub clamav_is_quarantine_repository_empty
{
  my @files = ();

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir(DIR);
  closedir (DIR);

  return (($#files - 1) <= 0);
}

# clamav_cut_string ( $ $ )
# IN: string, max length
# OUT: a cutted string
#
# Cut a string to fit in the given length, adding "[..]" at
# the end
# 
sub clamav_cut_string ( $ $ )
{   
  my ($string, $max_len) = @_;

  $string = (substr ($string, 0, $max_len - 5)) . ' [..]'
    if (($max_len > 5) and (length ($string) >= $max_len));
                                                                                
  return $string;
}

# clamav_is_clamd_installed ()
# IN: -
# OUT: -
#
# Check if specified clamd path is ok on the system
# 
sub clamav_is_clamd_installed
{
  return (-e $config{'clamav_init_script'});
}

# clamav_activate_clamd ()
# IN: -
# OUT: -
#
# Restart the clamav daemon, waiting on second for process
# to be up
# 
sub clamav_activate_clamd
{
  system ("$config{'clamav_init_script'} restart");
  sleep (1);
}

# clamav_deactivate_clamd ()
# IN: -
# OUT: -
#
# Stop the clamav daemon, waiting one second for process
# to be down
# 
sub clamav_deactivate_clamd
{
  system ("$config{'clamav_init_script'} stop");
  sleep (1);
}

# clamav_is_clamd_alive ()
# IN: -
# OUT: -
#
# Check wether or not clamd daemon is alive
# 
sub clamav_is_clamd_alive
{
  return &find_byname ($config{'clamav_daemon_name'});
}

# clamav_get_email_header_values ( $ $ )
# IN: email file name on the disk, field to retreive value for
# OUT: a array with requested values
#
# Search and return values in a array for the requested fields,
# in the inverse order than requested
# 
sub clamav_get_email_header_values ( $ $ )
{
  my ($file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;
 
  if (open (H, $file))
  {
    while (defined ($item = <H>) and ($i <= $#names))
    {
      foreach my $name (@names)
      {
        if ($item =~ /$name: .*\n$/)
        {
          $i++;
          $item =~ s/$name: //;
          $header{$name} = $item;
        }
      }
    }
    close (H);
  }
                                                                                
  return %header;
}

# clamav_get_log_values ( $ $ )
# IN: log file name on the disk, field to retreive value for
# OUT: a array with requested values
#
# Search and return values in a array for the requested fields,
# in the inverse order than requested
#
sub clamav_get_log_values ( $ $ )
{
  my ($file, @names) = @_;
  my %header = ();
  my $item = '';
  my $i = 0;
                                                                                
  if (open (H, $file))
  {
    while (defined ($item = <H>) and ($i <= $#names))
    {
      foreach my $name (@names)
      {
        if ($item =~ /: $name: .*\n$/)
        {
          $i++;
          $item =~ s/^.*: $name:.*: //;
          $header{$name} = $item;
        }
      }
    }
    close (H);
  }
                                                                                
  return %header;
}

# clamav_print_email ( $ )
# IN: email file on the disk
# OUT: -
#
# Display the content of the given mail file
# 
sub clamav_print_email ( $ )
{
  my $file = shift;
  my @content = ();
                                                                                
  exit (1) if ($file =~ /\// or $file =~ /\.\./ or $file =~ /;/);
    
  open (H, "$config{'clamav_quarantine'}/$file.msg");
  @content = <H>;
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_print_email_infos ( $ )
# IN: email file on the disk
# OUT: -
#
# Display some email information in a HTML table
# 
sub clamav_print_email_infos ( $ )
{
  my $base = shift;
  my %header = &clamav_get_email_header_values (
      "$config{'clamav_quarantine'}/$base.msg",
      qw(Subject X-Quarantined-From X-Quarantined-To)
  );
  my $subject = $header{'Subject'};
  my $from = $header{'X-Quarantined-From'};
  my $to = $header{'X-Quarantined-To'};

  print qq(<p><table border=0>\n);
  print qq(<tr><td $cb><b>$text{'SUBJECT'}: </b></td><td>$subject</td><tr>\n);
  print qq(<tr><td $cb><b>$text{'FROM'}: </b></td><td>$from</td><tr>\n);
  print qq(<tr><td $cb><b>$text{'TO'}: </b></td><td>$to</td><tr>\n);
  print qq(</table></p>\n);
}

# clamav_resend_email ( $ )
# IN: email file on the disk
# OUT: -
#
# Resend a false positive email. it first test if there is an
# existing amavis-inject script on the server. if not, it use
# it is proper copy of this perl script (./bin/amavis-inject).
#
# supported MTA are:
#
#   - rsmtp
#   - sendmail
# 
sub clamav_resend_email ( $ )
{
  my $base = shift;
  my $amavis = "amavis-inject ";
  my $mta = " sendmail -bS "; # use sendmail by default

  return if ($base =~ /\// or $base =~ /\.\./ or $base =~ /;/);
  
  # if rsmtp is found, use it instead of sendmail
  $mta = " rsmtp " if (&clamav_bin_exist ('rsmtp'));
  # if amavis-inject not found, use our own copy  
  $amavis = "./bin/amavis-inject " if (not &clamav_bin_exist ('amavis-inject'));

  system ("$amavis $config{'clamav_quarantine'}/$base.msg | $mta");
}

# clamav_remove_email ( $ )
# IN: base name of the email file on the disk
# OUT: -
#
# Delete a email and its associated log file on the quarantine
# repository
# 
sub clamav_remove_email
{
  my $base = shift;

  exit (1) if ($base =~ /\// or $base =~ /\.\./ or $base =~ /;/);
  
  system("/bin/rm -f $config{'clamav_quarantine'}/$base.*");
}

# clamav_print_log ( $ $ )
# IN: file to display, number of lines to display
# OUT: -
#
# Display the content of the given log file
#
sub clamav_print_log
{
  my $file = shift;
  my $lines = shift;
  my @content = ();

  exit (1) if ($file =~ /;/ or $lines =~ /;/);
  
  open (H, "$file");
  if ($lines eq $text{'ALL'})
  {
    @content = <H>;
  }
  else
  {
    @content = `/usr/bin/tail -n $lines $file`;
  }
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_quarantine_print_log ( $ )
# IN: email log file on the disk
# OUT: -
#
# Display the content of the given mail log file
#
sub clamav_quarantine_print_log ( $ )
{
  my $file = shift;
  my @content = ();

  exit (1) if ($file =~ /\// or $file =~ /\.\./ or $file =~ /;/);
    
  open (H, "$config{'clamav_quarantine'}/$file.log");
  @content = <H>;
  close (H);
                                                                                
  print qq(<textarea cols=80 rows=30>@content</textarea>\n);
}

# clamav_print_quarantine_table ()
# IN: -
# OUT: -
#
# Display a HTML table containing on line per email found in the quarantine
# repository
# 
sub clamav_print_quarantine_table
{
  my @files = ();
  my $i = 0;
  my $STRING_MAX_LEN = 30;

  opendir (DIR, $config{'clamav_quarantine'});
  @files = readdir (DIR);
  closedir (DIR);

  print qq(<table border>);
  print qq(<tr $tb><td><b>$text{'SUBJECT'}</b></td>);
  print qq(<td><b>$text{'VIRUS'}</b></td>);
  print qq(<td><b>$text{'FROM'}</b></td><td><b>$text{'TO'}</b></td>); 
  print qq(<td colspan=3><b>$text{'ACTION'}</b></td></tr>);

  for ($i = 0; $i < $#files + 1; $i += 2)
  {
    if ($files[$i] ne '.')
    {
      my $log = $files[$i];
      my $msg = $files[$i + 1];
      my $quarantine_path = $config{'clamav_quarantine'};
      my $base = $log; $base =~ s/\..*$//g;
      my %header = &clamav_get_email_header_values (
          "$quarantine_path/$base.msg",
          qw(Subject X-Quarantined-From X-Quarantined-To)
        );
      my $subject = $header{'Subject'};
      my $from = $header{'X-Quarantined-From'};
      my $to = $header{'X-Quarantined-To'};
      my %header = &clamav_get_log_values (
          "$quarantine_path/$base.log",
          qw(AMAVIS::AV::CLAMD)
        );
      my $virus = $header{'AMAVIS::AV::CLAMD'};
      my $url_virus = &urlize ($virus);

      $subject = &clamav_cut_string ($subject, $STRING_MAX_LEN);
      $from = &clamav_cut_string ($from, $STRING_MAX_LEN);
      $to = &clamav_cut_string ($to, $STRING_MAX_LEN);
      $virus = &clamav_cut_string ($virus, $STRING_MAX_LEN);

      print qq(<tr>\n);
      print qq(<td $cb><a href="quarantine_viewmail.cgi?base=$base">);
      print qq($subject</a></td>\n);
      print qq(<td $cb><b><a href="/clamav/vdb_search_main.cgi?search=on&virus=$url_virus">$virus</a></b></td>\n);
      print qq(<td $cb>$from</td>\n);
      print qq(<td $cb>$to</td>\n);
      print qq(<td $cb><a href="quarantine_viewlog.cgi?base=$base">);
      print qq($text{'VIEWLOG'}</a></td>\n);
      print qq(<td $cb><a href="quarantine_resend.cgi?base=$base">);
      print qq($text{'RESEND'}</a></td>\n);
      print qq(<td $cb><a href="quarantine_remove.cgi?base=$base">);
      print qq($text{'DELETE'}</a></td>\n);
      print qq(</tr>\n);
    }
  }

  print qq(</table>);
}
1;
